/****************************************************************************
 * (c) 2018 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * Helligkeit mit einem LDR messen
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include <Adafruit_ADS1015.h>
#include "arduino.h"
#include "ntc.h"
#include "i2c_adressen.h"

// Defines -------------------------------------------------------------------
// Umschalter fuer die Tests - ohne Kommentar = aktiv, in Kommentar = inaktiv
// Speicherbedarf
#define MITCALC
// Laufzeit
#define LAUFZEITMESSEN
// AD Wandler
#define I2C_CHAN_0       0x0
//#define MAX_ADC_READING  26666 // berechnet
#define MAX_ADC_READING  25530 // gemessen -> Messeingang auf +5V
// Laufzeitmessung
#define LZPIN 13
// Widerstand 
#define ADC_REF_VOLTAGE  5.0
#define REF_RESISTANCE   9340.0
// Widerstand bei 25 Grad Celsius
#define THERMISTORNOMINAL 10000      
// Temp. bei Nominal Widerstand (meist bei 25 C)
#define TEMPERATURENOMINAL 25   
// Der Beta Koeffizient vom Thermistor (ueblicherweise 3000-4000)
#define BCOEFFICIENT 3950
// Der Reihenwiderstand
#define SERIESRESISTOR 10000    

// Variablen -----------------------------------------------------------------
Adafruit_ADS1115 ads(I2C_SENSOR_ADC); // Standardeinstellung +- 6,144 Volt => 65535 bit (12,288 Volt)
                                      // 0 Volt bis +6,144 Volt => 32767 bit max
                                      // Spannungsteiler: 0 Volt - R mit 10 KOhm - Abgriff ADC - LDR - +5 Volt
                                      // MAX_ADC   = 5,0 Volt = 26666 bit
// Funktionen ----------------------------------------------------------------
void ntc_Setup(void)
{
  ads.setGain(GAIN_TWOTHIRDS); 
  ads.begin();
#ifdef LAUFZEITMESSEN
  pinMode(LZPIN, OUTPUT);
  digitalWrite(LZPIN, LOW);
#endif
}

signed int ntc_Read(void) // Sensor einlesen
{
  return(ads.readADC_SingleEnded(I2C_CHAN_0));
}

double ntc_CalcTemp(signed int adval)
{
#ifdef MITCALC
  double adcwert = 0.0;
  double steinhart = 0.0;

  adcwert   = ((double)(MAX_ADC_READING) / (double)adval) - 1.0;
  adcwert   = SERIESRESISTOR / adcwert;
  steinhart = THERMISTORNOMINAL / adcwert ;    // (R/Ro)
  steinhart = log(steinhart);                  // ln(R/Ro)
  steinhart /= BCOEFFICIENT;                   // 1/B * ln(R/Ro)
  steinhart += 1.0 / (TEMPERATURENOMINAL + 273.15); // + (1/To)
  steinhart = 1.0 / steinhart;                 // Invert
  steinhart -= 273.15;                         // convert to C
  return(steinhart);
#else
  return(0.0);
#endif
}

void ntc_BeginCalc(void)
{
#ifdef LAUFZEITMESSEN
  digitalWrite(LZPIN, HIGH);
#endif
}

void ntc_EndCalc(void)
{
#ifdef LAUFZEITMESSEN
  digitalWrite(LZPIN, LOW);
#endif  
}
